@@ -307,7 +307,6 @@ void PointCloudCommon::CloudInfo::clear()
 
 PointCloudCommon::PointCloudCommon(Display* display)
   : auto_size_(false)
-  , spinner_(1, &cbqueue_)
   , new_xyz_transformer_(false)
   , new_color_transformer_(false)
   , needs_retransform_(false)
@@ -377,18 +376,14 @@ void PointCloudCommon::initialize(DisplayContext* context, Ogre::SceneNode* scen
   updateBillboardSize();
   updateAlpha();
   updateSelectable();
-
-  spinner_.start();
 }
 
 PointCloudCommon::~PointCloudCommon()
 {
-  spinner_.stop();
-
-  if (transformer_class_loader_)
-  {
-    delete transformer_class_loader_;
-  }
+  // Ensure any threads holding the mutexes have finished
+  boost::recursive_mutex::scoped_lock lock1(transformers_mutex_);
+  boost::mutex::scoped_lock lock2(new_clouds_mutex_);
+  delete transformer_class_loader_;
 }
 
 void PointCloudCommon::loadTransformers()
