@@ -151,6 +151,7 @@ void RobotLinkSelectionHandler::postRenderPass(uint32_t /*pass*/)
   }
 }
 
+static std::map<const RobotLink*, std::string> errors;
 
 RobotLink::RobotLink(Robot* robot,
                      const urdf::LinkConstSharedPtr& link,
@@ -170,7 +171,7 @@ RobotLink::RobotLink(Robot* robot,
   , robot_alpha_(1.0)
   , only_render_depth_(false)
   , is_selectable_(true)
-  , using_color_(false)
+  , material_mode_flags_(ORIGINAL)
 {
   link_property_ = new Property(link->name.c_str(), true, "", nullptr, SLOT(updateVisibility()), this);
   link_property_->setIcon(rviz::loadPixmap("package://rviz/icons/classes/RobotLink.png"));
@@ -318,6 +319,26 @@ RobotLink::~RobotLink()
   delete axes_;
   delete details_;
   delete link_property_;
+  errors.erase(this);
+}
+
+void RobotLink::addError(const char* format, ...)
+{
+  char buffer[256];
+  va_list args;
+  va_start(args, format);
+  vsnprintf(buffer, sizeof(buffer), format, args);
+  va_end(args);
+
+  std::string& err = const_cast<std::string&>(getGeometryErrors());
+  if (!err.empty())
+    err.append("\n");
+  err.append(buffer);
+}
+
+const std::string& RobotLink::getGeometryErrors() const
+{
+  return errors[this];
 }
 
 bool RobotLink::hasGeometry() const
@@ -595,22 +616,22 @@ void RobotLink::createEntityForGeometryElement(const urdf::LinkConstSharedPtr& l
 
     scale = Ogre::Vector3(mesh.scale.x, mesh.scale.y, mesh.scale.z);
 
-    std::string model_name = mesh.filename;
+    const std::string& model_name = mesh.filename;
 
     try
     {
-      loadMeshFromResource(model_name);
-      entity = scene_manager_->createEntity(ss.str(), model_name);
+      if (loadMeshFromResource(model_name).isNull())
+        addError("Could not load mesh resource '%s'", model_name.c_str());
+      else
+        entity = scene_manager_->createEntity(ss.str(), model_name);
     }
     catch (Ogre::InvalidParametersException& e)
     {
-      ROS_ERROR("Could not convert mesh resource '%s' for link '%s'. It might be an empty mesh: %s",
-                model_name.c_str(), link->name.c_str(), e.what());
+      addError("Could not convert mesh resource '%s': %s", model_name.c_str(), e.what());
     }
     catch (Ogre::Exception& e)
     {
-      ROS_ERROR("Could not load model '%s' for link '%s': %s", model_name.c_str(), link->name.c_str(),
-                e.what());
+      addError("Could not load model '%s': %s", model_name.c_str(), e.what());
     }
     break;
   }
@@ -695,8 +716,8 @@ void RobotLink::createCollision(const urdf::LinkConstSharedPtr& link)
       if (collision_mesh)
       {
         collision_meshes_.push_back(collision_mesh);
-        valid_collision_found = true;
       }
+      valid_collision_found |= collision == link->collision; // don't consider the same geometry twice
     }
   }
 #endif
@@ -755,8 +776,8 @@ void RobotLink::createVisual(const urdf::LinkConstSharedPtr& link)
       if (visual_mesh)
       {
         visual_meshes_.push_back(visual_mesh);
-        valid_visual_found = true;
       }
+      valid_visual_found |= visual == link->visual; // don't consider the same geometry again
     }
   }
 #endif
@@ -879,38 +900,36 @@ void RobotLink::setTransforms(const Ogre::Vector3& visual_position,
 
 void RobotLink::setToErrorMaterial()
 {
-  for (size_t i = 0; i < visual_meshes_.size(); i++)
-  {
-    visual_meshes_[i]->setMaterialName("BaseWhiteNoLighting");
-  }
-  for (size_t i = 0; i < collision_meshes_.size(); i++)
-  {
-    collision_meshes_[i]->setMaterialName("BaseWhiteNoLighting");
-  }
+  setMaterialMode(material_mode_flags_ | ERROR);
 }
 
 void RobotLink::setToNormalMaterial()
 {
-  if (using_color_)
+  setMaterialMode(material_mode_flags_ & ~ERROR);
+}
+
+void RobotLink::setMaterialMode(unsigned char mode_flags)
+{
+  if (material_mode_flags_ == mode_flags)
+    return; // nothing to change
+
+  material_mode_flags_ = mode_flags;
+
+  if (mode_flags == ORIGINAL)
   {
-    for (size_t i = 0; i < visual_meshes_.size(); i++)
-    {
-      visual_meshes_[i]->setMaterial(color_material_);
-    }
-    for (size_t i = 0; i < collision_meshes_.size(); i++)
-    {
-      collision_meshes_[i]->setMaterial(color_material_);
-    }
-  }
-  else
-  {
-    M_SubEntityToMaterial::iterator it = materials_.begin();
-    M_SubEntityToMaterial::iterator end = materials_.end();
-    for (; it != end; ++it)
-    {
-      it->first->setMaterial(it->second);
-    }
+    for (const auto& item : materials_)
+      item.first->setMaterial(item.second);
+    return;
   }
+
+  auto error_material = Ogre::MaterialManager::getSingleton().getByName(
+      "BaseWhiteNoLighting", Ogre::ResourceGroupManager::INTERNAL_RESOURCE_GROUP_NAME);
+  auto material = mode_flags == COLOR ? color_material_ : error_material;
+
+  for (const auto& mesh : visual_meshes_)
+    mesh->setMaterial(material);
+  for (const auto& mesh : collision_meshes_)
+    mesh->setMaterial(material);
 }
 
 void RobotLink::setColor(float red, float green, float blue)
@@ -922,14 +941,12 @@ void RobotLink::setColor(float red, float green, float blue)
   color_material_->getTechnique(0)->setAmbient(0.5 * color);
   color_material_->getTechnique(0)->setDiffuse(color);
 
-  using_color_ = true;
-  setToNormalMaterial();
+  setMaterialMode(COLOR | (material_mode_flags_ & ERROR));
 }
 
 void RobotLink::unsetColor()
 {
-  using_color_ = false;
-  setToNormalMaterial();
+  setMaterialMode(ORIGINAL | (material_mode_flags_ & ERROR));
 }
 
 bool RobotLink::setSelectable(bool selectable)
